@@ -10,6 +10,7 @@ account, and subaccounting-excluding and -including balances.
 module Hledger.Data.Account
 where
 import Data.List
+import Data.Maybe
 import qualified Data.Map as M
 import Safe (headMay, lookupJustDef)
 import Test.HUnit
@@ -24,9 +25,10 @@ import Hledger.Utils
 
 -- deriving instance Show Account
 instance Show Account where
-    show Account{..} = printf "Account %s (boring:%s, ebalance:%s, ibalance:%s)"
-                       aname  
+    show Account{..} = printf "Account %s (boring:%s, postings:%d, ebalance:%s, ibalance:%s)"
+                       aname
                        (if aboring then "y" else "n" :: String)
+                       anumpostings
                        (showMixedAmount aebalance)
                        (showMixedAmount aibalance)
 
@@ -44,6 +46,7 @@ nullacct = Account
   { aname = ""
   , aparent = Nothing
   , asubs = []
+  , anumpostings = 0
   , aebalance = nullmixedamt
   , aibalance = nullmixedamt
   , aboring = False
@@ -57,10 +60,12 @@ accountsFromPostings ps =
   let
     acctamts = [(paccount p,pamount p) | p <- ps]
     grouped = groupBy (\a b -> fst a == fst b) $ sort $ acctamts
+    counted = [(a, length acctamts) | acctamts@((a,_):_) <- grouped]
     summed = map (\as@((aname,_):_) -> (aname, sum $ map snd as)) grouped -- always non-empty
     nametree = treeFromPaths $ map (expandAccountName . fst) summed
     acctswithnames = nameTreeToAccount "root" nametree
-    acctswithebals = mapAccounts setebalance acctswithnames where setebalance a = a{aebalance=lookupJustDef nullmixedamt (aname a) summed}
+    acctswithnumps = mapAccounts setnumps    acctswithnames where setnumps    a = a{anumpostings=fromMaybe 0 $ lookup (aname a) counted}
+    acctswithebals = mapAccounts setebalance acctswithnumps where setebalance a = a{aebalance=lookupJustDef nullmixedamt (aname a) summed}
     acctswithibals = sumAccounts acctswithebals
     acctswithparents = tieAccountParents acctswithibals
     acctsflattened = flattenAccounts acctswithparents
@@ -115,15 +120,52 @@ clipAccounts d a = a{asubs=subs}
     where
       subs = map (clipAccounts (d-1)) $ asubs a
 
+-- | Remove subaccounts below the specified depth, aggregating their balance at the depth limit
+-- (accounts at the depth limit will have any sub-balances merged into their exclusive balance).
+clipAccountsAndAggregate :: Int -> [Account] -> [Account]
+clipAccountsAndAggregate d as = combined
+    where
+      clipped  = [a{aname=clipAccountName d $ aname a} | a <- as]
+      combined = [a{aebalance=sum (map aebalance same)}
+                  | same@(a:_) <- groupBy (\a1 a2 -> aname a1 == aname a2) clipped]
+{-
+test cases, assuming d=1:
+
+assets:cash 1 1
+assets:checking 1 1
+->
+as:       [assets:cash 1 1, assets:checking 1 1]
+clipped:  [assets 1 1, assets 1 1]
+combined: [assets 2 2]
+
+assets 0 2
+ assets:cash 1 1
+ assets:checking 1 1
+->
+as:       [assets 0 2, assets:cash 1 1, assets:checking 1 1]
+clipped:  [assets 0 2, assets 1 1, assets 1 1]
+combined: [assets 2 2]
+
+assets 0 2
+ assets:bank 1 2
+  assets:bank:checking 1 1
+->
+as:       [assets 0 2, assets:bank 1 2, assets:bank:checking 1 1]
+clipped:  [assets 0 2, assets 1 2, assets 1 1]
+combined: [assets 2 2]
+
+-}
+
 -- | Remove all leaf accounts and subtrees matching a predicate.
 pruneAccounts :: (Account -> Bool) -> Account -> Maybe Account
 pruneAccounts p = headMay . prune
   where
     prune a
-      | null prunedsubs = if p a then [] else [a]
-      | otherwise       = [a{asubs=prunedsubs}]
+      | null prunedsubs = if p a then [] else [a']
+      | otherwise       = [a']
       where
         prunedsubs = concatMap prune $ asubs a
+        a' = a{asubs=prunedsubs}
 
 -- | Flatten an account tree into a list, which is sometimes
 -- convenient. Note since accounts link to their parents/subs, the
