@@ -1,3 +1,4 @@
+-- {-# LANGUAGE CPP #-}
 {-|
 
 A 'Journal' is a set of transactions, plus optional related data.  This is
@@ -19,8 +20,10 @@ module Hledger.Data.Journal (
   journalConvertAmountsToCost,
   journalFinalise,
   -- * Filtering
-  filterJournalPostings,
   filterJournalTransactions,
+  filterJournalPostings,
+  filterJournalPostingAmounts,
+  filterPostingAmount,
   -- * Querying
   journalAccountNames,
   journalAccountNamesUsed,
@@ -28,6 +31,7 @@ module Hledger.Data.Journal (
   journalAmounts,
   -- journalCanonicalCommodities,
   journalDateSpan,
+  journalDescriptions,
   journalFilePath,
   journalFilePaths,
   journalPostings,
@@ -153,13 +157,19 @@ addHistoricalPrice h l0 = l0 { historical_prices = h : historical_prices l0 }
 addTimeLogEntry :: TimeLogEntry -> Journal -> Journal
 addTimeLogEntry tle l0 = l0 { open_timelog_entries = tle : open_timelog_entries l0 }
 
+-- | Unique transaction descriptions used in this journal.
+journalDescriptions :: Journal -> [String]
+journalDescriptions = nub . sort . map tdescription . jtxns
+
+-- | All postings from this journal's transactions, in order.
 journalPostings :: Journal -> [Posting]
 journalPostings = concatMap tpostings . jtxns
 
--- | All account names used in this journal.
+-- | Unique account names posted to in this journal.
 journalAccountNamesUsed :: Journal -> [AccountName]
 journalAccountNamesUsed = sort . accountNamesFromPostings . journalPostings
 
+-- | Unique account names in this journal, including parent accounts containing no postings.
 journalAccountNames :: Journal -> [AccountName]
 journalAccountNames = sort . expandAccountNames . journalAccountNamesUsed
 
@@ -228,6 +238,17 @@ filterJournalPostings q j@Journal{jtxns=ts} = j{jtxns=map filtertransactionposti
     where
       filtertransactionpostings t@Transaction{tpostings=ps} = t{tpostings=filter (q `matchesPosting`) ps}
 
+-- Within each posting's amount, keep only the parts matching the query.
+-- This can leave unbalanced transactions.
+filterJournalPostingAmounts :: Query -> Journal -> Journal
+filterJournalPostingAmounts q j@Journal{jtxns=ts} = j{jtxns=map filtertransactionpostings ts}
+    where
+      filtertransactionpostings t@Transaction{tpostings=ps} = t{tpostings=map (filterPostingAmount q) ps}
+
+-- | Filter out all parts of this posting's amount which do not match the query.
+filterPostingAmount :: Query -> Posting -> Posting
+filterPostingAmount q p@Posting{pamount=Mixed as} = p{pamount=Mixed $ filter (q `matchesAmount`) as}
+
 -- | Keep only transactions matching the query expression.
 filterJournalTransactions :: Query -> Journal -> Journal
 filterJournalTransactions q j@Journal{jtxns=ts} = j{jtxns=filter (q `matchesTransaction`) ts}
@@ -415,10 +436,10 @@ checkBalanceAssertion (errs,bal) ps
     assertion = pbalanceassertion p
     Just assertedbal = assertion
     bal' = sum $ [bal] ++ map pamount ps
-    err = printf "Balance assertion failed for account %s on %s\n%safter\n   %s\nexpected balance is %s, actual balance was %s."
+    err = printf "Balance assertion failed for account %s on %s\n%sAfter posting:\n   %s\nexpected balance is %s, actual balance was %s."
                  (paccount p)
                  (show $ postingDate p)
-                 (maybe "" (("In\n"++).show) $ ptransaction p)
+                 (maybe "" (("In transaction:\n"++).show) $ ptransaction p)
                  (show p)
                  (showMixedAmount assertedbal)
                  (showMixedAmount bal')
@@ -524,14 +545,35 @@ journalMixedAmounts = map pamount . journalPostings
 journalAmounts :: Journal -> [Amount]
 journalAmounts = concatMap flatten . journalMixedAmounts where flatten (Mixed as) = as
 
--- | The (fully specified) date span containing this journal's transactions,
--- or DateSpan Nothing Nothing if there are none.
-journalDateSpan :: Journal -> DateSpan
-journalDateSpan j
-    | null ts = DateSpan Nothing Nothing
-    | otherwise = DateSpan (Just $ tdate $ head ts) (Just $ addDays 1 $ tdate $ last ts)
+-- | The fully specified date span enclosing the dates (primary or secondary)
+-- of all this journal's transactions and postings, or DateSpan Nothing Nothing
+-- if there are none.
+journalDateSpan :: Bool -> Journal -> DateSpan
+journalDateSpan secondary j
+    | null ts   = DateSpan Nothing Nothing
+    | otherwise = DateSpan (Just earliest) (Just $ addDays 1 latest)
     where
-      ts = sortBy (comparing tdate) $ jtxns j
+      earliest = minimum dates
+      latest   = maximum dates
+      dates    = pdates ++ tdates
+      tdates   = map (if secondary then transactionDate2 else tdate) ts
+      pdates   = concatMap (catMaybes . map (if secondary then (Just . postingDate2) else pdate) . tpostings) ts
+      ts       = jtxns j
+
+-- #ifdef TESTS
+test_journalDateSpan = do
+ "journalDateSpan" ~: do
+  assertEqual "" (DateSpan (Just $ fromGregorian 2014 1 10) (Just $ fromGregorian 2014 10 11))
+                 (journalDateSpan True j)
+  where
+    j = nulljournal{jtxns = [nulltransaction{tdate = parsedate "2014/02/01"
+                                            ,tpostings = [posting{pdate=Just (parsedate "2014/01/10")}]
+                                            }
+                            ,nulltransaction{tdate = parsedate "2014/09/01"
+                                            ,tpostings = [posting{pdate2=Just (parsedate "2014/10/10")}]
+                                            }
+                            ]}
+-- #endif
 
 -- Misc helpers
 
@@ -662,6 +704,7 @@ Right samplejournal = journalBalanceTransactions $
 
 tests_Hledger_Data_Journal = TestList $
  [
+  test_journalDateSpan
   -- "query standard account types" ~:
   --  do
   --   let j = journal1
