@@ -35,16 +35,17 @@ nullledger = Ledger {
   laccounts = []
   }
 
--- | Filter a journal's transactions with the given query, then derive a
--- ledger containing the chart of accounts and balances. If the query
--- includes a depth limit, that will affect the ledger's journal but not
--- the account tree.
+-- | Filter a journal's transactions with the given query, then derive
+-- a ledger containing the chart of accounts and balances. If the
+-- query includes a depth limit, that will affect the this ledger's
+-- journal but not the ledger's account tree.
 ledgerFromJournal :: Query -> Journal -> Ledger
 ledgerFromJournal q j = nullledger{ljournal=j'', laccounts=as}
   where
     (q',depthq)  = (filterQuery (not . queryIsDepth) q, filterQuery queryIsDepth q)
-    j' = filterJournalPostings q' j
-    as = accountsFromPostings $ journalPostings j'
+    j'  = filterJournalPostingAmounts (filterQuery queryIsSym q) $ -- remove amount parts which the query's sym: terms would exclude
+          filterJournalPostings q' j
+    as  = accountsFromPostings $ journalPostings j'
     j'' = filterJournalPostings depthq j'
 
 -- | List a ledger's account names.
