@@ -16,9 +16,11 @@ module Hledger.Query (
   filterQuery,
   -- * accessors
   queryIsNull,
+  queryIsAcct,
   queryIsDepth,
   queryIsDate,
   queryIsStartDateOnly,
+  queryIsSym,
   queryStartDate,
   queryDateSpan,
   queryDepth,
@@ -26,9 +28,12 @@ module Hledger.Query (
   inAccount,
   inAccountQuery,
   -- * matching
-  matchesAccount,
-  matchesPosting,
   matchesTransaction,
+  matchesPosting,
+  matchesAccount,
+  matchesMixedAmount,
+  matchesAmount,
+  words'',
   -- * tests
   tests_Hledger_Query
 )
@@ -45,7 +50,7 @@ import Text.ParserCombinators.Parsec
 import Hledger.Utils
 import Hledger.Data.Types
 import Hledger.Data.AccountName
-import Hledger.Data.Amount (amount, usd)
+import Hledger.Data.Amount (amount, nullamt, usd)
 import Hledger.Data.Dates
 import Hledger.Data.Posting
 import Hledger.Data.Transaction
@@ -65,7 +70,7 @@ data Query = Any              -- ^ always match
            | Date2 DateSpan   -- ^ match if secondary date in this date span
            | Status Bool      -- ^ match if cleared status has this value
            | Real Bool        -- ^ match if "realness" (involves a real non-virtual account ?) has this value
-           | Amt Ordering Quantity   -- ^ match if the amount's numeric quantity is less than/greater than/equal to some value
+           | Amt OrdPlus Quantity  -- ^ match if the amount's numeric quantity is less than/greater than/equal to/unsignedly equal to some value
            | Sym String       -- ^ match if the entire commodity symbol is matched by this regexp
            | Empty Bool       -- ^ if true, show zero-amount postings/accounts which are usually not shown
                               --   more of a query option than a query criteria ?
@@ -165,18 +170,17 @@ tests_parseQuery = [
 words'' :: [String] -> String -> [String]
 words'' prefixes = fromparse . parsewith maybeprefixedquotedphrases -- XXX
     where
-      maybeprefixedquotedphrases = choice' [prefixedQuotedPattern, quotedPattern, pattern] `sepBy` many1 spacenonewline
+      maybeprefixedquotedphrases = choice' [prefixedQuotedPattern, singleQuotedPattern, doubleQuotedPattern, pattern] `sepBy` many1 spacenonewline
       prefixedQuotedPattern = do
         not' <- fromMaybe "" `fmap` (optionMaybe $ string "not:")
         let allowednexts | null not' = prefixes
                          | otherwise = prefixes ++ [""]
         next <- choice' $ map string allowednexts
         let prefix = not' ++ next
-        p <- quotedPattern
+        p <- singleQuotedPattern <|> doubleQuotedPattern
         return $ prefix ++ stripquotes p
-      quotedPattern = do
-        p <- between (oneOf "'\"") (oneOf "'\"") $ many $ noneOf "'\""
-        return $ stripquotes p
+      singleQuotedPattern = between (char '\'') (char '\'') (many $ noneOf "'") >>= return . stripquotes
+      doubleQuotedPattern = between (char '"') (char '"') (many $ noneOf "\"") >>= return . stripquotes
       pattern = many (noneOf " \n\r")
 
 tests_words'' = [
@@ -204,7 +208,7 @@ prefixes = map (++":") [
     ,"date"
     ,"edate"
     ,"status"
-    ,"sym"
+    ,"cur"
     ,"real"
     ,"empty"
     ,"depth"
@@ -241,10 +245,10 @@ parseQueryTerm d ('e':'d':'a':'t':'e':':':s) =
                                     Right (_,span) -> Left $ Date2 span
 parseQueryTerm _ ('s':'t':'a':'t':'u':'s':':':s) = Left $ Status $ parseStatus s
 parseQueryTerm _ ('r':'e':'a':'l':':':s) = Left $ Real $ parseBool s
-parseQueryTerm _ ('a':'m':'t':':':s) = Left $ Amt op q where (op, q) = parseAmountQueryTerm s
+parseQueryTerm _ ('a':'m':'t':':':s) = Left $ Amt ord q where (ord, q) = parseAmountQueryTerm s
 parseQueryTerm _ ('e':'m':'p':'t':'y':':':s) = Left $ Empty $ parseBool s
 parseQueryTerm _ ('d':'e':'p':'t':'h':':':s) = Left $ Depth $ readDef 0 s
-parseQueryTerm _ ('s':'y':'m':':':s) = Left $ Sym s
+parseQueryTerm _ ('c':'u':'r':':':s) = Left $ Sym s -- support cur: as an alias
 parseQueryTerm _ ('t':'a':'g':':':s) = Left $ Tag n v where (n,v) = parseTag s
 parseQueryTerm _ "" = Left $ Any
 parseQueryTerm d s = parseQueryTerm d $ defaultprefix++":"++s
@@ -269,25 +273,43 @@ tests_parseQueryTerm = [
     -- "amt:>10000.10" `gives` (Left $ Amt GT 10000.1)
  ]
 
+
+data OrdPlus = Lt | Gt | Eq | AbsLt | AbsGt | AbsEq
+ deriving (Show,Eq,Data,Typeable)
+
 -- can fail
-parseAmountQueryTerm :: String -> (Ordering, Quantity)
-parseAmountQueryTerm s =
-  case s of
-    ""     -> err
-    '<':s' -> (LT, readDef err s')
-    '=':s' -> (EQ, readDef err s')
-    '>':s' -> (GT, readDef err s')
-    s'     -> (EQ, readDef err s')
+parseAmountQueryTerm :: String -> (OrdPlus, Quantity)
+parseAmountQueryTerm s' =
+  case s' of
+    -- feel free to do this a smarter way
+    ""        -> err
+    '<':'+':s -> (Lt, readDef err s)
+    '>':'+':s -> (Gt, readDef err s)
+    '=':'+':s -> (Eq, readDef err s)
+    '+':s     -> (Eq, readDef err s)
+    '<':'-':s -> (Lt, negate $ readDef err s)
+    '>':'-':s -> (Gt, negate $ readDef err s)
+    '=':'-':s -> (Eq, negate $ readDef err s)
+    '-':s     -> (Eq, negate $ readDef err s)
+    '<':s     -> let n = readDef err s in case n of 0 -> (Lt, 0)
+                                                    _ -> (AbsLt, n)
+    '>':s     -> let n = readDef err s in case n of 0 -> (Gt, 0)
+                                                    _ -> (AbsGt, n)
+    '=':s     -> (AbsEq, readDef err s)
+    s         -> (AbsEq, readDef err s)
   where
-    err = error' $ "could not parse as '=', '<', or '>' (optional) followed by a numeric quantity: " ++ s
+    err = error' $ "could not parse as '=', '<', or '>' (optional) followed by a (optionally signed) numeric quantity: " ++ s'
 
 tests_parseAmountQueryTerm = [
   "parseAmountQueryTerm" ~: do
     let s `gives` r = parseAmountQueryTerm s `is` r
-    "<0" `gives` (LT,0)
-    "=0.23" `gives` (EQ,0.23)
-    "0.23" `gives` (EQ,0.23)
-    ">10000.10" `gives` (GT,10000.1)
+    "<0" `gives` (Lt,0) -- special case for convenience, since AbsLt 0 would be always false
+    ">0" `gives` (Gt,0) -- special case for convenience and consistency with above
+    ">10000.10" `gives` (AbsGt,10000.1)
+    "=0.23" `gives` (AbsEq,0.23)
+    "0.23" `gives` (AbsEq,0.23)
+    "=+0.23" `gives` (Eq,0.23)
+    "-0.23" `gives` (Eq,(-0.23))
   ]
 
 parseTag :: String -> (String, Maybe String)
@@ -381,6 +403,7 @@ queryIsDepth _ = False
 
 queryIsDate :: Query -> Bool
 queryIsDate (Date _) = True
+queryIsDate (Date2 _) = True
 queryIsDate _ = False
 
 queryIsDesc :: Query -> Bool
@@ -391,6 +414,10 @@ queryIsAcct :: Query -> Bool
 queryIsAcct (Acct _) = True
 queryIsAcct _ = False
 
+queryIsSym :: Query -> Bool
+queryIsSym (Sym _) = True
+queryIsSym _ = False
+
 -- | Does this query specify a start date and nothing else (that would
 -- filter postings prior to the date) ?
 -- When the flag is true, look for a starting secondary date instead.
@@ -517,6 +544,35 @@ tests_matchesAccount = [
     assertBool "" $ not $ (Tag "a" Nothing) `matchesAccount` "a"
  ]
 
+matchesMixedAmount :: Query -> MixedAmount -> Bool
+matchesMixedAmount q (Mixed []) = q `matchesAmount` nullamt
+matchesMixedAmount q (Mixed as) = any (q `matchesAmount`) as
+
+-- | Does the match expression match this (simple) amount ?
+matchesAmount :: Query -> Amount -> Bool
+matchesAmount (Not q) a = not $ q `matchesAmount` a
+matchesAmount (Any) _ = True
+matchesAmount (None) _ = False
+matchesAmount (Or qs) a = any (`matchesAmount` a) qs
+matchesAmount (And qs) a = all (`matchesAmount` a) qs
+--
+matchesAmount (Amt ord n) a = compareAmount ord n a
+matchesAmount (Sym r) a = regexMatchesCI ("^" ++ r ++ "$") $ acommodity a
+--
+matchesAmount _ _ = True
+
+-- | Is this simple (single-amount) mixed amount's quantity less than, greater than, equal to, or unsignedly equal to this number ?
+-- For multi-amount (multiple commodities, or just unsimplified) mixed amounts this is always true.
+
+-- | Is this amount's quantity less than, greater than, equal to, or unsignedly equal to this number ?
+compareAmount :: OrdPlus -> Quantity -> Amount -> Bool
+compareAmount ord q Amount{aquantity=aq} = case ord of Lt    -> aq <  q
+                                                       Gt    -> aq >  q
+                                                       Eq    -> aq == q
+                                                       AbsLt -> abs aq <  abs q
+                                                       AbsGt -> abs aq >  abs q
+                                                       AbsEq -> abs aq == abs q
+
 -- | Does the match expression match this posting ?
 matchesPosting :: Query -> Posting -> Bool
 matchesPosting (Not q) p = not $ q `matchesPosting` p
@@ -531,8 +587,9 @@ matchesPosting (Date span) p = span `spanContainsDate` postingDate p
 matchesPosting (Date2 span) p = span `spanContainsDate` postingDate2 p
 matchesPosting (Status v) p = v == postingCleared p
 matchesPosting (Real v) p = v == isReal p
-matchesPosting (Depth d) Posting{paccount=a} = Depth d `matchesAccount` a
-matchesPosting (Amt op n) Posting{pamount=a} = compareMixedAmount op n a
+matchesPosting q@(Depth _) Posting{paccount=a} = q `matchesAccount` a
+matchesPosting q@(Amt _ _) Posting{pamount=amt} = q `matchesMixedAmount` amt
+-- matchesPosting q@(Amt _ _) Posting{pamount=amt} = q `matchesMixedAmount` amt
 -- matchesPosting (Empty v) Posting{pamount=a} = v == isZeroMixedAmount a
 -- matchesPosting (Empty False) Posting{pamount=a} = True
 -- matchesPosting (Empty True) Posting{pamount=a} = isZeroMixedAmount a
@@ -542,14 +599,6 @@ matchesPosting (Tag n Nothing) p = isJust $ lookupTagByName n $ postingAllTags p
 matchesPosting (Tag n (Just v)) p = isJust $ lookupTagByNameAndValue (n,v) $ postingAllTags p
 -- matchesPosting _ _ = False
 
--- | Is this simple mixed amount's quantity less than, equal to, or greater than this number ?
--- For complext mixed amounts (with multiple commodities), this is always true.
-compareMixedAmount :: Ordering -> Quantity -> MixedAmount -> Bool
-compareMixedAmount op q (Mixed [])  = compareMixedAmount op q (Mixed [amount])
--- compareMixedAmount op q (Mixed [a]) = strace (compare (strace $ aquantity a) (strace q)) == op
-compareMixedAmount op q (Mixed [a]) = compare (aquantity a) q == op
-compareMixedAmount _ _ _            = True
-
 tests_matchesPosting = [
    "matchesPosting" ~: do
     -- matching posting status..
@@ -613,8 +662,8 @@ tests_matchesTransaction = [
    assertBool "" $ (Desc "x x") `matchesTransaction` nulltransaction{tdescription="x x"}
    -- see posting for more tag tests
    assertBool "" $ (Tag "foo" (Just "a")) `matchesTransaction` nulltransaction{ttags=[("foo","bar")]}
-   -- a tag match on a transaction usually ignores posting tags
-   assertBool "" $ not $ (Tag "postingtag" Nothing) `matchesTransaction` nulltransaction{tpostings=[nullposting{ptags=[("postingtag","")]}]}
+   -- a tag match on a transaction also matches posting tags
+   assertBool "" $ (Tag "postingtag" Nothing) `matchesTransaction` nulltransaction{tpostings=[nullposting{ptags=[("postingtag","")]}]}
  ]
 
 lookupTagByName :: String -> [Tag] -> Maybe Tag
