@@ -26,13 +26,16 @@ module Hledger.Read.JournalReader (
   directive,
   defaultyeardirective,
   historicalpricedirective,
-  datetime,
-  code,
-  accountname,
+  datetimep,
+  codep,
+  accountnamep,
+  postingp,
   amountp,
   amountp',
   mamountp',
-  emptyline
+  numberp,
+  emptyorcommentlinep,
+  followingcommentp
 #ifdef TESTS
   -- * Tests
   -- disabled by default, HTF not available on windows
@@ -158,7 +161,7 @@ journal = do
                            , liftM (return . addModifierTransaction) modifiertransaction
                            , liftM (return . addPeriodicTransaction) periodictransaction
                            , liftM (return . addHistoricalPrice) historicalpricedirective
-                           , emptyline >> return (return id)
+                           , emptyorcommentlinep >> return (return id)
                            ] <?> "journal transaction or directive"
 
 -- cf http://ledger-cli.org/3.0/doc/ledger3.html#Command-Directives
@@ -206,7 +209,7 @@ accountdirective :: GenParser Char JournalContext JournalUpdate
 accountdirective = do
   string "account"
   many1 spacenonewline
-  parent <- accountname
+  parent <- accountnamep
   newline
   pushParentAccount parent
   return $ return id
@@ -271,7 +274,7 @@ historicalpricedirective :: GenParser Char JournalContext HistoricalPrice
 historicalpricedirective = do
   char 'P' <?> "historical price"
   many spacenonewline
-  date <- try (do {LocalTime d _ <- datetime; return d}) <|> date -- a time is ignored
+  date <- try (do {LocalTime d _ <- datetimep; return d}) <|> date -- a time is ignored
   many1 spacenonewline
   symbol <- commoditysymbol
   many spacenonewline
@@ -322,9 +325,9 @@ transaction = do
   date <- date <?> "transaction"
   edate <- optionMaybe (secondarydate date) <?> "secondary date"
   status <- status <?> "cleared flag"
-  code <- code <?> "transaction code"
+  code <- codep <?> "transaction code"
   description <- descriptionp >>= return . strip
-  comment <- try followingcomment <|> (newline >> return "")
+  comment <- try followingcommentp <|> (newline >> return "")
   let tags = tagsInComment comment
   postings <- postings
   return $ txnTieKnot $ Transaction date edate status code description comment tags postings ""
@@ -442,8 +445,8 @@ date = do
 -- timezone will be ignored; the time is treated as local time.  Fewer
 -- digits are allowed, except in the timezone. The year may be omitted if
 -- a default year has already been set.
-datetime :: GenParser Char JournalContext LocalTime
-datetime = do
+datetimep :: GenParser Char JournalContext LocalTime
+datetimep = do
   day <- date
   many1 spacenonewline
   h <- many1 digit
@@ -486,8 +489,8 @@ secondarydate primarydate = do
 status :: GenParser Char JournalContext Bool
 status = try (do { many spacenonewline; (char '*' <|> char '!') <?> "status"; return True } ) <|> return False
 
-code :: GenParser Char JournalContext String
-code = try (do { many1 spacenonewline; char '(' <?> "code"; code <- anyChar `manyTill` char ')'; return code } ) <|> return ""
+codep :: GenParser Char JournalContext String
+codep = try (do { many1 spacenonewline; char '(' <?> "codep"; code <- anyChar `manyTill` char ')'; return code } ) <|> return ""
 
 -- Parse the following whitespace-beginning lines as postings, posting tags, and/or comments.
 postings :: GenParser Char JournalContext [Posting]
@@ -512,7 +515,7 @@ postingp = do
   _ <- fixedlotprice
   many spacenonewline
   ctx <- getState
-  comment <- try followingcomment <|> (newline >> return "")
+  comment <- try followingcommentp <|> (newline >> return "")
   let tags = tagsInComment comment
   -- oh boy
   d  <- maybe (return Nothing) (either (fail.show) (return.Just)) (parseWithCtx ctx date `fmap` dateValueFromTags tags)
@@ -569,7 +572,7 @@ test_postingp = do
 -- | Parse an account name, then apply any parent account prefix and/or account aliases currently in effect.
 modifiedaccountname :: GenParser Char JournalContext AccountName
 modifiedaccountname = do
-  a <- accountname
+  a <- accountnamep
   prefix <- getParentAccount
   let prefixed = prefix `joinAccountNames` a
   aliases <- getAccountAliases
@@ -579,12 +582,12 @@ modifiedaccountname = do
 -- them, and are terminated by two or more spaces. They should have one or
 -- more components of at least one character, separated by the account
 -- separator char.
-accountname :: GenParser Char st AccountName
-accountname = do
+accountnamep :: GenParser Char st AccountName
+accountnamep = do
     a <- many1 (nonspace <|> singlespace)
     let a' = striptrailingspace a
     when (accountNameFromComponents (accountNameComponents a') /= a')
-         (fail $ "accountname seems ill-formed: "++a')
+         (fail $ "account name seems ill-formed: "++a')
     return a'
     where 
       singlespace = try (do {spacenonewline; do {notFollowedBy spacenonewline; return ' '}})
@@ -646,21 +649,27 @@ amountp' s = either (error' . show) id $ parseWithCtx nullctx amountp s
 mamountp' :: String -> MixedAmount
 mamountp' = mixed . amountp'
 
+signp :: GenParser Char JournalContext String
+signp = do
+  sign <- optionMaybe $ oneOf "+-"
+  return $ case sign of Just '-' -> "-"
+                        _        -> ""
+
 leftsymbolamount :: GenParser Char JournalContext Amount
 leftsymbolamount = do
-  sign <- optionMaybe $ string "-"
-  let applysign = if isJust sign then negate else id
+  sign <- signp
   c <- commoditysymbol 
   sp <- many spacenonewline
-  (q,prec,dec,sep,seppos) <- number
+  (q,prec,dec,sep,seppos) <- numberp
   let s = amountstyle{ascommodityside=L, ascommodityspaced=not $ null sp, asdecimalpoint=dec, asprecision=prec, asseparator=sep, asseparatorpositions=seppos}
   p <- priceamount
+  let applysign = if sign=="-" then negate else id
   return $ applysign $ Amount c q p s
   <?> "left-symbol amount"
 
 rightsymbolamount :: GenParser Char JournalContext Amount
 rightsymbolamount = do
-  (q,prec,dec,sep,seppos) <- number
+  (q,prec,dec,sep,seppos) <- numberp
   sp <- many spacenonewline
   c <- commoditysymbol
   p <- priceamount
@@ -670,12 +679,12 @@ rightsymbolamount = do
 
 nosymbolamount :: GenParser Char JournalContext Amount
 nosymbolamount = do
-  (q,prec,dec,sep,seppos) <- number
+  (q,prec,dec,sep,seppos) <- numberp
   p <- priceamount
   defcs <- getCommodityAndStyle
   let (c,s) = case defcs of
-        Just (c',s') -> (c',s')
-        Nothing -> ("", amountstyle{asdecimalpoint=dec, asprecision=prec, asseparator=sep, asseparatorpositions=seppos})
+        Just (defc,defs) -> (defc, defs{asprecision=max (asprecision defs) prec})
+        Nothing          -> ("", amountstyle{asdecimalpoint=dec, asprecision=prec, asseparator=sep, asseparatorpositions=seppos})
   return $ Amount c q p s
   <?> "no-symbol amount"
 
@@ -743,25 +752,26 @@ fixedlotprice =
 -- and separator characters (defaulting to . and ,), and the positions of
 -- separators (counting leftward from the decimal point, the last is
 -- assumed to repeat).
-number :: GenParser Char JournalContext (Quantity, Int, Char, Char, [Int])
-number = do
-  sign <- optionMaybe $ string "-"
+numberp :: GenParser Char JournalContext (Quantity, Int, Char, Char, [Int])
+numberp = do
+  sign <- signp
   parts <- many1 $ choice' [many1 digit, many1 $ char ',', many1 $ char '.']
   let numeric = isNumber . headDef '_'
-      (_, puncparts) = partition numeric parts
+      (numparts, puncparts) = partition numeric parts
       (ok,decimalpoint',separator') =
-          case puncparts of
-            []     -> (True, Nothing, Nothing)  -- no punctuation chars
-            [d:""] -> (True, Just d, Nothing)   -- just one punctuation char, assume it's a decimal point
-            [_]    -> (False, Nothing, Nothing) -- adjacent punctuation chars, not ok
-            _:_:_  -> let (s:ss, d) = (init puncparts, last puncparts) -- two or more punctuation chars
-                     in if (any ((/=1).length) puncparts  -- adjacent punctuation chars, not ok
-                            || any (s/=) ss                -- separator chars differ, not ok
-                            || head parts == s)            -- number begins with a separator char, not ok
-                         then (False, Nothing, Nothing)
-                         else if s == d
-                               then (True, Nothing, Just $ head s) -- just one kind of punctuation, assume separator chars
-                               else (True, Just $ head d, Just $ head s) -- separators and a decimal point
+          case (numparts,puncparts) of
+            ([],_)     -> (False, Nothing, Nothing)  -- no digits
+            (_,[])     -> (True, Nothing, Nothing)  -- no punctuation chars
+            (_,[d:""]) -> (True, Just d, Nothing)   -- just one punctuation char, assume it's a decimal point
+            (_,[_])    -> (False, Nothing, Nothing) -- adjacent punctuation chars, not ok
+            (_,_:_:_)  -> let (s:ss, d) = (init puncparts, last puncparts) -- two or more punctuation chars
+                          in if (any ((/=1).length) puncparts  -- adjacent punctuation chars, not ok
+                                 || any (s/=) ss                -- separator chars differ, not ok
+                                 || head parts == s)            -- number begins with a separator char, not ok
+                              then (False, Nothing, Nothing)
+                              else if s == d
+                                    then (True, Nothing, Just $ head s) -- just one kind of punctuation, assume separator chars
+                                    else (True, Just $ head d, Just $ head s) -- separators and a decimal point
   when (not ok) (fail $ "number seems ill-formed: "++concat parts)
   let (intparts',fracparts') = span ((/= decimalpoint') . Just . head) parts
       (intparts, fracpart) = (filter numeric intparts', filter numeric fracparts')
@@ -771,8 +781,7 @@ number = do
       precision = length frac
       int' = if null int then "0" else int
       frac' = if null frac then "0" else frac
-      sign' = fromMaybe "" sign
-      quantity = read $ sign'++int'++"."++frac' -- this read should never fail
+      quantity = read $ sign++int'++"."++frac' -- this read should never fail
       (decimalpoint, separator) = case (decimalpoint', separator') of (Just d,  Just s)   -> (d,s)
                                                                       (Just '.',Nothing)  -> ('.',',')
                                                                       (Just ',',Nothing)  -> (',','.')
@@ -780,12 +789,12 @@ number = do
                                                                       (Nothing, Just ',') -> ('.',',')
                                                                       _                   -> ('.',',')
   return (quantity,precision,decimalpoint,separator,separatorpositions)
-  <?> "number"
+  <?> "numberp"
 
 #ifdef TESTS
-test_number = do
-      let s `is` n = assertParseEqual' (parseWithCtx nullctx number s) n
-          assertFails = assertBool . isLeft . parseWithCtx nullctx number 
+test_numberp = do
+      let s `is` n = assertParseEqual' (parseWithCtx nullctx numberp s) n
+          assertFails = assertBool . isLeft . parseWithCtx nullctx numberp 
       assertFails ""
       "0"          `is` (0, 0, '.', ',', [])
       "1"          `is` (1, 0, '.', ',', [])
@@ -808,23 +817,28 @@ test_number = do
 
 -- comment parsers
 
-emptyline :: GenParser Char JournalContext ()
-emptyline = do many spacenonewline
-               optional $ (char ';' <?> "comment") >> many (noneOf "\n")
-               newline
-               return ()
+emptyorcommentlinep :: GenParser Char JournalContext ()
+emptyorcommentlinep = do
+  many spacenonewline >> (comment <|> (many spacenonewline >> newline >> return ""))
+  return ()
 
-followingcomment :: GenParser Char JournalContext String
-followingcomment =
-  -- ptrace "followingcomment"
-  do samelinecomment <- many spacenonewline >> (try commentline <|> (newline >> return ""))
-     newlinecomments <- many (try (many1 spacenonewline >> commentline))
+followingcommentp :: GenParser Char JournalContext String
+followingcommentp =
+  -- ptrace "followingcommentp"
+  do samelinecomment <- many spacenonewline >> (try semicoloncomment <|> (newline >> return ""))
+     newlinecomments <- many (try (many1 spacenonewline >> semicoloncomment))
      return $ unlines $ samelinecomment:newlinecomments
 
-commentline :: GenParser Char JournalContext String
-commentline = do
-  -- ptrace "commentline"
-  char ';'
+comment :: GenParser Char JournalContext String
+comment = commentStartingWith "#;"
+
+semicoloncomment :: GenParser Char JournalContext String
+semicoloncomment = commentStartingWith ";"
+
+commentStartingWith :: String -> GenParser Char JournalContext String
+commentStartingWith cs = do
+  -- ptrace "commentStartingWith"
+  oneOf cs
   many spacenonewline
   l <- anyChar `manyTill` eolof
   optional newline
@@ -892,12 +906,12 @@ date2ValueFromTags ts = maybe Nothing (Just . snd) $ find ((=="date2") . fst) ts
 {- old hunit tests
 
 test_Hledger_Read_JournalReader = TestList $ concat [
-    test_number,
+    test_numberp,
     test_amountp,
     test_spaceandamountormissing,
     test_tagcomment,
     test_inlinecomment,
-    test_commentlines,
+    test_comments,
     test_ledgerDateSyntaxToTags,
     test_postingp,
     test_transaction,
@@ -913,18 +927,18 @@ test_Hledger_Read_JournalReader = TestList $ concat [
      assertParse (parseWithCtx nullctx directive "account some:account\n")
      assertParse (parseWithCtx nullctx (directive >> directive) "!account a\nend\n")
 
-  ,"commentline" ~: do
-     assertParse (parseWithCtx nullctx commentline "; some comment \n")
-     assertParse (parseWithCtx nullctx commentline " \t; x\n")
-     assertParse (parseWithCtx nullctx commentline ";x")
+  ,"comment" ~: do
+     assertParse (parseWithCtx nullctx comment "; some comment \n")
+     assertParse (parseWithCtx nullctx comment " \t; x\n")
+     assertParse (parseWithCtx nullctx comment "#x")
 
   ,"date" ~: do
      assertParse (parseWithCtx nullctx date "2011/1/1")
      assertParseFailure (parseWithCtx nullctx date "1/1")
      assertParse (parseWithCtx nullctx{ctxYear=Just 2011} date "1/1")
 
-  ,"datetime" ~: do
-      let p = do {t <- datetime; eof; return t}
+  ,"datetimep" ~: do
+      let p = do {t <- datetimep; eof; return t}
           bad = assertParseFailure . parseWithCtx nullctx p
           good = assertParse . parseWithCtx nullctx p
       bad "2011/1/1"
@@ -962,11 +976,11 @@ test_Hledger_Read_JournalReader = TestList $ concat [
      assertParse (parseWithCtx nullctx endtagdirective "end tag \n")
      assertParse (parseWithCtx nullctx endtagdirective "pop \n")
 
-  ,"accountname" ~: do
-    assertBool "accountname parses a normal accountname" (isRight $ parsewith accountname "a:b:c")
-    assertBool "accountname rejects an empty inner component" (isLeft $ parsewith accountname "a::c")
-    assertBool "accountname rejects an empty leading component" (isLeft $ parsewith accountname ":b:c")
-    assertBool "accountname rejects an empty trailing component" (isLeft $ parsewith accountname "a:b:")
+  ,"accountnamep" ~: do
+    assertBool "accountnamep parses a normal account name" (isRight $ parsewith accountnamep "a:b:c")
+    assertBool "accountnamep rejects an empty inner component" (isLeft $ parsewith accountnamep "a::c")
+    assertBool "accountnamep rejects an empty leading component" (isLeft $ parsewith accountnamep ":b:c")
+    assertBool "accountnamep rejects an empty trailing component" (isLeft $ parsewith accountnamep "a:b:")
 
   ,"leftsymbolamount" ~: do
     assertParseEqual (parseWithCtx nullctx leftsymbolamount "$1")  (usd 1 `withPrecision` 0)
